@@ -363,6 +363,32 @@ pub(crate) fn parse(text: &str) -> Parse {
 // Type aliases for convenience
 type SyntaxNode = rowan::SyntaxNode<Lang>;
 
+/// Calculate line and column (both 0-indexed) for the given offset in the tree.
+/// Column is measured in bytes from the start of the line.
+fn line_col_at_offset(node: &SyntaxNode, offset: rowan::TextSize) -> (usize, usize) {
+    let root = node.ancestors().last().unwrap_or_else(|| node.clone());
+    let mut line = 0;
+    let mut last_newline_offset = rowan::TextSize::from(0);
+
+    for element in root.preorder_with_tokens() {
+        if let rowan::WalkEvent::Enter(rowan::NodeOrToken::Token(token)) = element {
+            if token.text_range().start() >= offset {
+                break;
+            }
+
+            // Count newlines and track position of last one
+            for (idx, _) in token.text().match_indices('\n') {
+                line += 1;
+                last_newline_offset =
+                    token.text_range().start() + rowan::TextSize::from((idx + 1) as u32);
+            }
+        }
+    }
+
+    let column: usize = (offset - last_newline_offset).into();
+    (line, column)
+}
+
 /// The root of an INI/.desktop file
 #[derive(Debug, Clone, PartialEq, Eq, Hash)]
 pub struct Desktop(SyntaxNode);
@@ -393,6 +419,22 @@ impl Desktop {
         let text = std::fs::read_to_string(path)?;
         Self::from_str(&text)
     }
+
+    /// Get the line number (0-indexed) where this node starts.
+    pub fn line(&self) -> usize {
+        line_col_at_offset(&self.0, self.0.text_range().start()).0
+    }
+
+    /// Get the column number (0-indexed, in bytes) where this node starts.
+    pub fn column(&self) -> usize {
+        line_col_at_offset(&self.0, self.0.text_range().start()).1
+    }
+
+    /// Get both line and column (0-indexed) where this node starts.
+    /// Returns (line, column) where column is measured in bytes from the start of the line.
+    pub fn line_col(&self) -> (usize, usize) {
+        line_col_at_offset(&self.0, self.0.text_range().start())
+    }
 }
 
 impl AstNode for Desktop {
@@ -584,6 +626,22 @@ impl Group {
     pub fn syntax(&self) -> &SyntaxNode {
         &self.0
     }
+
+    /// Get the line number (0-indexed) where this node starts.
+    pub fn line(&self) -> usize {
+        line_col_at_offset(&self.0, self.0.text_range().start()).0
+    }
+
+    /// Get the column number (0-indexed, in bytes) where this node starts.
+    pub fn column(&self) -> usize {
+        line_col_at_offset(&self.0, self.0.text_range().start()).1
+    }
+
+    /// Get both line and column (0-indexed) where this node starts.
+    /// Returns (line, column) where column is measured in bytes from the start of the line.
+    pub fn line_col(&self) -> (usize, usize) {
+        line_col_at_offset(&self.0, self.0.text_range().start())
+    }
 }
 
 impl AstNode for Group {
@@ -691,6 +749,22 @@ impl Entry {
     pub fn syntax(&self) -> &SyntaxNode {
         &self.0
     }
+
+    /// Get the line number (0-indexed) where this node starts.
+    pub fn line(&self) -> usize {
+        line_col_at_offset(&self.0, self.0.text_range().start()).0
+    }
+
+    /// Get the column number (0-indexed, in bytes) where this node starts.
+    pub fn column(&self) -> usize {
+        line_col_at_offset(&self.0, self.0.text_range().start()).1
+    }
+
+    /// Get both line and column (0-indexed) where this node starts.
+    /// Returns (line, column) where column is measured in bytes from the start of the line.
+    pub fn line_col(&self) -> (usize, usize) {
+        line_col_at_offset(&self.0, self.0.text_range().start())
+    }
 }
 
 impl AstNode for Entry {
@@ -962,4 +1036,53 @@ Name=Example
         assert_eq!(group.get_locale("Name", "fr"), Some("Exemple".to_string()));
         assert_eq!(group.get_locales("Name").len(), 2);
     }
+
+    #[test]
+    fn test_line_col() {
+        let text = r#"[Desktop Entry]
+Name=Example Application
+Type=Application
+Exec=example
+
+[Desktop Action Play]
+Name=Play
+Exec=example --play
+"#;
+        let desktop = Desktop::from_str(text).unwrap();
+
+        // Test desktop root starts at line 0
+        assert_eq!(desktop.line(), 0);
+        assert_eq!(desktop.column(), 0);
+
+        // Test group line numbers
+        let groups: Vec<_> = desktop.groups().collect();
+        assert_eq!(groups.len(), 2);
+
+        // First group starts at line 0
+        assert_eq!(groups[0].line(), 0);
+        assert_eq!(groups[0].column(), 0);
+
+        // Second group starts at line 5 (after empty line)
+        assert_eq!(groups[1].line(), 5);
+        assert_eq!(groups[1].column(), 0);
+
+        // Test entry line numbers in first group
+        let entries: Vec<_> = groups[0].entries().collect();
+        assert_eq!(entries[0].line(), 1); // Name=Example Application
+        assert_eq!(entries[1].line(), 2); // Type=Application
+        assert_eq!(entries[2].line(), 3); // Exec=example
+
+        // Test column numbers
+        assert_eq!(entries[0].column(), 0); // Start of line
+        assert_eq!(entries[1].column(), 0); // Start of line
+
+        // Test line_col() method
+        assert_eq!(groups[1].line_col(), (5, 0));
+        assert_eq!(entries[0].line_col(), (1, 0));
+
+        // Test entries in second group
+        let second_group_entries: Vec<_> = groups[1].entries().collect();
+        assert_eq!(second_group_entries[0].line(), 6); // Name=Play
+        assert_eq!(second_group_entries[1].line(), 7); // Exec=example --play
+    }
 }
